
/******************************************************************************
* MODULE     : variant.hpp
* DESCRIPTION: Many Mmxlib types (vectors, polynomials, series, ...)
*              admit several "variants" according to their representation and
*              the algorithms for the fundamental operations (multiplication).
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_VARIANT_HPP
#define __MML_VARIANT_HPP
namespace mml {

/******************************************************************************
* Basic kinds of representations
******************************************************************************/

template<typename V>
class dense {
public:
  static const nat def_len = 0;
  static const nat def_rows= 0;
  static const nat def_cols= 0;
};

template<typename V, nat n> class linear {};
template<typename V, nat n>
class dense<linear<V,n> > {
public:
  static const nat def_len = n;
  static const nat def_rows= 0;
  static const nat def_cols= 0;
};

template<typename V, nat rows, nat cols> class rectangular {};
template<typename V, nat rows, nat cols>
class dense<rectangular<V,rows,cols> > {
public:
  static const nat def_len = 0;
  static const nat def_rows= rows;
  static const nat def_cols= cols;
};

template<typename V> class sparse {};

/******************************************************************************
* Basic kinds of multiplication algorithms
******************************************************************************/

class naive {};
template<nat threshold> class karatsuba {};
// we should still think about the fft
template<typename Op> class skew {};

} // namespace mml
#endif // __MML_VARIANT_HPP
