
/******************************************************************************
* MODULE     : interval_out.cpp
* DESCRIPTION: Outputting intervals
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

ostream&
operator << (ostream& out, const interval& x) {
  if (is_error (x)) return out << "NanI";
  if (is_fuzzy (x)) return out << "Fuzz";

  // return out << "[ " << lower (x) << ", " << upper (x) << " ]";

  dyadic mid= center (x);
  dyadic rad= radius (x);
  if (is_zero (rad)) return out << mid;
  if (is_zero (mid)) return out << dyadic (mid, MIN_PREC);
  mp_exp_t bits= max ((mp_exp_t) expo (mid) - expo (rad) - 1,
		      (mp_exp_t) MIN_PREC);
  return print (out, *mid, min (prec (mid), bits));
}

ostream&
print_lisp (ostream& out, const interval& x) {
  return out << x;
}

} // namespace mml
