
/******************************************************************************
* MODULE     : real_add.cpp
* DESCRIPTION: Addition of real numbers
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class add_real_rep: public real_rep {
private:
  real x, y;
public:
  inline add_real_rep (real x2, real y2):
    real_rep (weight (x2, y2)), x (x2), y (y2) {}
  generic expression () const {
    return plus (as_generic (x), as_generic (y)); }
  dyadic get_bound () { return add_up (bound (x), bound (y)); }
  interval approx (const dyadic& err);
};

interval
add_real_rep::approx (const dyadic& err) {
  if (wider (best, err)) {
    interval xa= approximate (x, spread (err, x->wt, wt));
    interval ya= approximate (y, spread (err, y->wt, wt));
    mp_prec_t new_prec= linear_algo (err);
    best= reduce (xa, new_prec) + reduce (ya, new_prec);
    ENSURE_REAL_CONSISTENCY (err);
  }
  return best;
}

real
operator + (const real& x, const real& y) {
  return real (new add_real_rep (x, y));
}

}
