
/******************************************************************************
* MODULE     : real_neg.cpp
* DESCRIPTION: Compute the opposite of a real number
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class neg_real_rep: public real_rep {
private:
  real x;
public:
  inline neg_real_rep (real x2):
    real_rep (weight (x2)), x (x2) {}
  generic expression () const { return negate (as_generic (x)); }
  dyadic get_bound () { return bound (x); }
  interval approx (const dyadic& err);
};

interval
neg_real_rep::approx (const dyadic& err) {
  if (wider (best, err)) {
    interval xa= approximate (x, err);
    mp_prec_t new_prec= linear_algo (err);
    best= -reduce (xa, new_prec);
    ENSURE_REAL_CONSISTENCY (err);
  }
  return best;
}

real
operator - (const real& x) {
  return real (new neg_real_rep (x));
}

}
