
/******************************************************************************
* MODULE     : real_sint.cpp
* DESCRIPTION: Signed integers as effective real numbers
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
#include "integer.hpp"
namespace mml {

class signed_int_real_rep: public real_rep {
private:
  signed long int i;
public:
  inline signed_int_real_rep (signed long int i2):
    real_rep (dyadic (1, DEFAULT_LOW), i2), i (i2) {}
  generic expression () const { return as_generic (i); }
  dyadic get_bound () { return abs (dyadic (i)); }
  interval approx (const dyadic& err);
};

interval
signed_int_real_rep::approx (const dyadic& err) {
  if (wider (best, err) && (i != 0)) {
    mp_prec_t new_prec= linear_algo (err);
    best= extend (best, new_prec);
  }
  return best;
}

real::real (signed int i): rep (new signed_int_real_rep (i)) {}
real::real (signed short int i): rep (new signed_int_real_rep (i)) {}
real::real (signed long int i): rep (new signed_int_real_rep (i)) {}

}
