
/******************************************************************************
* MODULE     : real_uint.cpp
* DESCRIPTION: Unsigned integers as real numbers
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "real.hpp"
namespace mml {

class unsigned_int_real_rep: public real_rep {
private:
  unsigned long int i;
public:
  inline unsigned_int_real_rep (unsigned long int i2):
    real_rep (dyadic (1, DEFAULT_LOW), i2), i (i2) {}
  generic expression () const { return as_generic (i); }
  dyadic get_bound () { return dyadic (i); }
  interval approx (const dyadic& err);
};

interval
unsigned_int_real_rep::approx (const dyadic& err) {
  if (wider (best, err) && (i != 0)) {
    mp_prec_t new_prec= linear_algo (err);
    best= extend (best, new_prec);
  }
  return best;
}

real::real (unsigned int i): rep (new unsigned_int_real_rep (i)) {}
real::real (unsigned short int i): rep (new unsigned_int_real_rep (i)) {}
real::real (unsigned long int i): rep (new unsigned_int_real_rep (i)) {}

}
