
/******************************************************************************
* MODULE     : series_test.cc
* DESCRIPTION: Test series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rational.hpp"
#include "series.hpp"
using namespace mml;

rational fun (nat n) { return 1; }

int
main () {
  series<rational> f (5, 3);
  series<rational> g (fun, "fun");
  series<rational> h= seq (rational (3), rational (6), rational (2));

  // cout << set_series_expr (true);
  cout << "f\t= " << f << "\n";
  cout << "g\t= " << g << "\n";
  cout << "h\t= " << h << "\n";
  cout << "-f\t= " << -f << "\n";
  cout << "f+g\t= " << f+g << "\n";
  cout << "f-g\t= " << f-g << "\n";
  cout << "g-f\t= " << g-f << "\n";
  cout << "f*g\t= " << f*g << "\n";
  cout << "2*f\t= " << 2*f << "\n";
  cout << "f*2\t= " << f*2 << "\n";
  cout << "f/g\t= " << f/g << "\n";
  cout << "f/3\t= " << f/3 << "\n";
  cout << "f+=g\t= " << (f+=g) << "\n";
  cout << "f-=g\t= " << (f-=g) << "\n";
  cout << "f*=g\t= " << (f*=g) << "\n";
  cout << "f/=g\t= " << (f/=g) << "\n";
  cout << "f*=11\t= " << (f*=11) << "\n";
  cout << "f/=11\t= " << (f/=11) << "\n";
  cout << "f<<2\t= " << (f<<2) << "\n";
  cout << "f>>2\t= " << (f>>2) << "\n";
  cout << "g<<2\t= " << (g<<2) << "\n";
  cout << "g>>2\t= " << (g>>2) << "\n";
  cout << "g o z^3\t= " << dilate (g, 3) << "\n";
  cout << "g'\t= " << derive (g) << "\n";
  cout << "`g\t= " << integrate (g) << "\n";
  cout << "f=g\t= " << (f==g) << "\n";
  cout << "f!=g\t= " << (f!=g) << "\n";
  cout << "f<g\t= " << (f<g) << "\n";
  cout << "f<=g\t= " << (f<=g) << "\n";
  cout << "f>g\t= " << (f>g) << "\n";
  cout << "f>=g\t= " << (f>=g) << "\n\n";

  cout << "1/(1+f)\t\t= " << 1/(1+f)<<"\n";
  cout << "exp(f)\t\t= " << exp (f) << "\n";
  cout << "log(1+f)\t= " << log (1+f) << "\n";
  cout << "exp(z*exp(f))\t= " << exp (exp (f) << 1) << "\n";
  cout << "sqrt(1+f)\t= " << sqrt (1+f) << "\n";
  cout << "(1+f)^7\t\t= " << pow (1+f, rational(7)) << "\n";
  cout << "sin(f)\t\t= " << sin (f) << "\n";
  cout << "cos(f)\t\t= " << cos (f) << "\n";
  cout << "tan(f)\t\t= " << tan (f) << "\n\n";

  cout << "------------------------------------------------------\n\n";

  series<integer> zero (0);
  series<integer> one (1);
  series<integer> z (1, 1);
  series<integer> a= one / (one - z);
  series<integer> b= a * a;

  cout << "a\t\t= " << a << "\n";
  cout << "b\t\t= " << b << "\n";
  cout << "b / z\t\t= " << b / z << "\n";
  cout << "b / (2 + z)\t= " << b / (2 + z) << "\n";
  cout << "b / 0\t\t= " << b / zero << "\n";
  cout << "b % z\t\t= " << b % z << "\n";
  cout << "b % (2 + z)\t= " << b % (2 + z) << "\n";
  cout << "b % 0\t\t= " << b % zero << "\n";

  return 0;
}
