
/******************************************************************************
* MODULE     : skew_polynomial.hpp
* DESCRIPTION: Skew polynomials
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_SKEW_POLYNOMIAL_HPP
#define __MML_SKEW_POLYNOMIAL_HPP
#include "dense_polynomial.hpp"
#include "skew_jet.hpp"
#include "generic_symbols.hpp"
namespace mml {

/******************************************************************************
* The operator d/dx
******************************************************************************/

template<typename C> generic
var (const polynomial<C,dense<skew<d_operator> > >& P) {
  (void) P;
  return MML_PARTIAL;
}

template<typename C> polynomial<C,dense<skew<d_operator> > >
adjoint (const polynomial<C,dense<skew<d_operator> > >& P) {
  nat n= dim (P);
  if (n <= 1) return P;
  C* r= new C[n];
  jet_dicho_adjoint<C,d_operator> (r, seg (P), n, false);
  return polynomial<C,dense<skew<d_operator> > > (r, n);
}

/******************************************************************************
* The operator x d/dx
******************************************************************************/

template<typename C> generic
var (const polynomial<C,dense<skew<xd_operator> > >& P) {
  (void) P;
  return MML_DELTA;
}

template<typename C> polynomial<C,dense<skew<xd_operator> > >
adjoint (const polynomial<C,dense<skew<xd_operator> > >& P) {
  nat n= dim (P);
  if (n <= 1) return P;
  C* r= new C[n];
  jet_dicho_adjoint<C,xd_operator> (r, seg (P), n, false);
  return polynomial<C,dense<skew<xd_operator> > > (r, n);
}

/******************************************************************************
* The shift operators x |-> x + 1 and x |-> x - 1
******************************************************************************/

template<typename C, typename K> generic
var (const polynomial<C,dense<skew<S_operator<K> > > >& P) {
  (void) P;
  return "S";
}

template<typename C, typename K> generic
var (const polynomial<C,dense<skew<T_operator<K> > > >& P) {
  (void) P;
  return "T";
}

template<typename C, typename K>
polynomial<C,dense<skew<S_operator<K> > > >
adjoint (const polynomial<C,dense<skew<S_operator<K> > > >& P) {
  nat n= dim (P);
  C* r= new C[n];
  if (n>1) jet_dicho_adjoint<C,S_operator<K> > (r, seg (P), n, false);
  return polynomial<C,dense<skew<S_operator<K> > > > (r, n);
}

template<typename C, typename K>
polynomial<C,dense<skew<T_operator<K> > > >
adjoint (const polynomial<C,dense<skew<T_operator<K> > > >& P) {
  nat n= dim (P);
  C* r= new C[n];
  if (n>1) jet_dicho_adjoint<C,T_operator<K> > (r, seg (P), n, false);
  return polynomial<C,dense<skew<T_operator<K> > > > (r, n);
}

} // namespace mml
#endif // __MML_SKEW_POLYNOMIAL_HPP
