
/******************************************************************************
* MODULE     : symbolic.hpp
* DESCRIPTION: Symbolic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIC_HPP
#define __SYMBOLIC_HPP
#include "universe.hpp"
#include "generic_internal.hpp"
namespace mml {
#define TMPL_DEF template<typename C,typename V=rules<exp_universe> >
#define TMPL template<typename C,typename V>
#define Symbolic symbolic<C,V>
#define Vector vector<Symbolic >
#define Table table<Symbolic,Symbolic >
TMPL class symbolic_rep;
TMPL class symbolic;

/******************************************************************************
* Symbolic class
******************************************************************************/

TMPL_DEF
class symbolic {
private:
  generic rep;

public:
  symbolic (const generic& g): rep (g) {}
  generic operator * () const { return rep; }

  symbolic ();
  symbolic (int i);
  symbolic (const C& sc);
  symbolic (char* s);
  symbolic (const string& s);
  symbolic (const Vector& v);

  Symbolic operator () () const;
  Symbolic operator () (const Symbolic& x1) const;
  Symbolic operator () (const Symbolic& x1, const Symbolic& x2) const;
  Symbolic operator () (const Symbolic& x1, const Symbolic& x2,
			const Symbolic& x3) const;
  Symbolic operator [] (nat i) const { return rep[i]; }
};

/******************************************************************************
* Constructors
******************************************************************************/

TMPL generic constructor (const C& sc) {
  return generic_scalar<C> (sc); }
TMPL generic constructor (const string& s) {
  return generic_atom (s); }
TMPL generic constructor (const Vector& v) {
  return generic_compound (*reinterpret_cast<const vector<generic>*> (&v)); }

TMPL Symbolic::symbolic (): rep (constructor<C,V> (C(0))) {}
TMPL Symbolic::symbolic (int i): rep (constructor<C,V> (C(i))) {}
TMPL Symbolic::symbolic (const C& sc): rep (constructor<C,V> (sc)) {}
TMPL Symbolic::symbolic (char* s): rep (constructor<C,V> (string (s))) {}
TMPL Symbolic::symbolic (const string& s): rep (constructor<C,V> (s)) {}
TMPL Symbolic::symbolic (const Vector& v): rep (constructor<C,V> (v)) {}

TMPL Symbolic Symbolic::operator () () const {
  return constructor (vec (*this)); }
TMPL Symbolic Symbolic::operator () (const Symbolic& x1) const {
  return constructor (vec (*this, x1)); }
TMPL Symbolic Symbolic::operator ()
  (const Symbolic& x1, const Symbolic& x2) const {
    return constructor (vec (*this, x1, x2)); }
TMPL Symbolic Symbolic::operator ()
  (const Symbolic& x1, const Symbolic& x2, const Symbolic& x3) const {
    return constructor (vec (*this, x1, x2, x3)); }

TMPL Symbolic symbolic_sum (const Table& t) {
  return generic_sum (
    *reinterpret_cast<const table<generic,generic>*> (&t)); }
TMPL Symbolic symbolic_product (const Table& t) {
  return generic_product (
    *reinterpret_cast<const table<generic,generic>*> (&t)); }

/******************************************************************************
* Predicates and inspection
******************************************************************************/

TMPL bool operator == (const Symbolic& x1, const Symbolic& x2) {
  return (*x1) == (*x2); }
TMPL bool operator != (const Symbolic& x1, const Symbolic& x2) {
  return (*x1) != (*x2); }
TMPL bool operator == (const Symbolic& x, int i) {
  return (*x) == constructor<C,V> (C(i)); }
TMPL bool operator != (const Symbolic& x, int i) {
  return (*x) != constructor<C,V> (C(i)); }

TMPL bool is_scalar (const Symbolic& x) { return is_scalar (*x); }
TMPL bool is_atom (const Symbolic& x) { return is_atom (*x); }
TMPL bool is_compound (const Symbolic& x) { return is_compound (*x); }
TMPL bool is_sum (const Symbolic& x) { return is_sum (*x); }
TMPL bool is_product (const Symbolic& x) { return is_product (*x); }
TMPL bool is_func (const Symbolic& x, const Symbolic& f, nat i) {
    return is_func (*x, f.rep, i); }
TMPL bool is_func (const Symbolic& x, char* s, nat i) {
    return is_func (*x, s, i); }
TMPL bool is_func (const Symbolic& x, const string& s, nat i) {
    return is_func (*x, s, i); }

TMPL nat N (const Symbolic& x) { return N (*x); }
TMPL nat hash (const Symbolic& x) { return hash (*x); }

/******************************************************************************
* Conversions
******************************************************************************/

TMPL C as_scalar (const Symbolic& x) {
  return as_scalar<C> (*x); }
TMPL string as_string (const Symbolic& x) {
  return as_string (*x); }

TMPL Vector as_vector (const Symbolic& x) {
  vector<generic> v= as_vector (*x);
  return *reinterpret_cast<Vector*> (&v); }
TMPL vector<generic> as_vector_generic (const Vector& v) {
  return *reinterpret_cast<const vector<generic>*> (&v); }

TMPL Table as_sum (const Symbolic& x) {
  table<generic,generic> t= as_sum (*x);
  return *reinterpret_cast<Table*> (&t); }
TMPL Table as_product (const Symbolic& x) {
  table<generic,generic> t= as_product (*x);
  return *reinterpret_cast<Table*> (&t); }
TMPL table<generic,generic> as_table_generic (const Table& t) {
  return *reinterpret_cast<const table<generic,generic>*> (&t); }

TMPL generic as_generic (const Symbolic& x) {
  return *x; }
TMPL ostream& operator << (ostream& out, const Symbolic& x) {
  return out << (*x); }

#undef TMPL_DEF
#undef TMPL
#undef Symbolic
#undef Vector
#undef Table
} // namespace mml
#endif // __SYMBOLIC_HPP
