
/******************************************************************************
* MODULE     : generic_compound.cpp
* DESCRIPTION: Compound generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
namespace mml {

class generic_compound_rep: public generic_rep {
public:
  symbol<vector<generic> > data;

protected:
  nat get_type () const { return GENERIC_COMPOUND; }
  nat get_dim () const { return dim (*data); }
  generic get_child (nat i) const { return (*data) [i]; }
  nat get_hash_value () const { return hash (data); }
  bool identical (const generic& g) const {
    if (type (g) != GENERIC_COMPOUND) return false;
    return data == ((generic_compound_rep*) inspect (g)) -> data; }
  generic flatten (bool recursive) const {
    vector<generic> v= *data;
    nat i, n= dim (v);
    vector<generic> w= vector<generic> (generic (), n);
    for (i=0; i<n; i++)
      w(i)= mml::flatten (v[i], recursive);
    return generic_compound (w); }

public:
  generic_compound_rep (const vector<generic>& data2): data (data2) {}
};

generic
generic::operator () () const {
  return new generic_compound_rep (vec (*this));
}

generic
generic::operator () (const generic& g1) const {
  return new generic_compound_rep (vec (*this, g1));
}

generic
generic::operator () (const generic& g1, const generic& g2) const {
  return new generic_compound_rep (vec (*this, g1, g2));
}

generic
generic::operator () (const generic& g1, const generic& g2,
		      const generic& g3) const {
  return new generic_compound_rep (vec (*this, g1, g2, g3));
}

generic
generic_compound (const vector<generic>& v) {
  return new generic_compound_rep (v);
}

generic
generic_compound (const generic& g1) {
  return new generic_compound_rep (vec (g1));
}

generic
generic_compound (const generic& g1, const generic& g2) {
  return new generic_compound_rep (vec (g1, g2));
}

generic
generic_compound (const generic& g1, const generic& g2, const generic& g3) {
  return new generic_compound_rep (vec (g1, g2, g3));
}

symbol<vector<generic> >
as_compound (const generic& g) {
  return ((generic_compound_rep*) inspect (g)) -> data;
}

vector<generic>
as_vector (const generic& g) {
  return * ((generic_compound_rep*) inspect (g)) -> data;
}

} // namespace mml
