
/******************************************************************************
* MODULE     : generic_product.cpp
* DESCRIPTION: Power products of generics represented by hash tables
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
#include "list_sort.hpp"
namespace mml {

class generic_product_rep: public generic_rep {
public:
  symbol<table<generic,generic> > data;

protected:
  nat get_type () const { return GENERIC_PRODUCT; }
  nat get_dim () const { return 0; }
  generic get_child (nat i) const {
    assert (false); // illegal access
    return generic (); }
  nat get_hash_value () const { return hash (data); }
  bool identical (const generic& g) const {
    if (type (g) != GENERIC_PRODUCT) return false;
    return data == ((generic_product_rep*) inspect (g)) -> data; }
  generic flatten (bool recursive) const {
    table<generic,generic> sum= *data;
    list<generic> l (entries (sum));
    l= sort (l, small_big_compare);
    generic acc= 1;
    while (!nil (l)) {
      generic mon  = mml::flatten (car (l), recursive);
      generic coeff= mml::flatten (sum[car (l)], recursive);
      acc= times (acc, power (mon, coeff, true), true);
      l= cdr (l);
    }
    return acc;
  }

public:
  generic_product_rep (const table<generic,generic>& data2): data (data2) {}
};

generic
generic_product (const table<generic,generic>& t) {
  return new generic_product_rep (t);
}

table<generic,generic>
as_product (const generic& g) {
  return * ((generic_product_rep*) inspect (g)) -> data;
}

} // namespace mml
