
/******************************************************************************
* MODULE     : generic_utils.cpp
* DESCRIPTION: Utility functions for generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
namespace mml {

/******************************************************************************
* Size function
******************************************************************************/

nat
size (const generic& g) {
  if (is_atom (g)) return 1;
  if (is_compound (g)) {
    nat i, n= N (g), sum= 1;
    for (i=0; i<n; i++)
      sum += size (g[i]);
    return sum;
  }
  return size (flatten (g));
}

/******************************************************************************
* Compairison
******************************************************************************/

static bool
is_numeric (const string& s) {
  return
    (N(s) > 0 && s[0] >= '0' && s[0] <= '9') ||
    (N(s) > 1 && s[0] == '-' && s[1] >= '0' && s[1] <= '9');
}

static int
compare_numeric (const string& s1, const string& s2) {
  if (s1[0] == '-' && s2[0] != '-') return -1;
  if (s1[0] != '-' && s2[0] == '-') return 1;
  if (s1[0] == '-' && s2[0] == '-')
    return -compare_numeric (s1 (1, N (s1)), s2 (1, N(s2)));
  if (N(s1) < N(s2)) return -1;
  if (N(s1) > N(s2)) return 1;
  if (s1 < s2) return -1;
  if (s1 > s2) return 1;
  return 0;
}

static int
compare (const generic& g1, const generic& g2, int direction) {
  nat s1= size (g1), s2= size (g2);
  if (s1 < s2) return -direction;
  if (s1 > s2) return direction;
  if (is_atom (g1) and is_atom (g2)) {
    string l1= as_string (g1), l2= as_string (g2);
    if (is_numeric (l1) && !is_numeric (l2)) return -direction;
    if (is_numeric (l2) && !is_numeric (l1)) return direction;
    if (is_numeric (l1) && is_numeric (l2))
      return direction * compare_numeric (l1, l2);
    if (l1 < l2) return -1;
    if (l1 > l2) return 1;
    return 0;
  }
  if (is_atom (g1) && is_compound (g2)) return -direction;
  if (is_compound (g1) && is_atom (g2)) return direction;
  if (is_compound (g1) && is_compound (g2)) {
    nat i, n1= N (g1), n2= N (g2);
    if (n1 < n2) return -direction;
    if (n1 > n2) return direction;
    for (i=0; i<n1; i++) {
      int c= compare (g1[i], g2[i], direction);
      if (c != 0) return c;
    }
    return 0;
  }
  return compare (flatten (g1), flatten (g2), direction);
}

int
big_small_compare (const generic& g1, const generic& g2) {
  return compare (g1, g2, -1);
}

int
small_big_compare (const generic& g1, const generic& g2) {
  return compare (g1, g2, 1);
}

} // namespace mml
